@@ -1,32 +1,18 @@
-from __future__ import annotations
-
 import functools
 import hashlib
-import itertools
-import time
-from enum import Enum
+from dataclasses import dataclass
 from itertools import chain
+from typing import Protocol
 
 import gitlab.const
-from attrs import define
 from github import Auth, Consts, Github, GithubException, PullRequest
-from github.GithubException import UnknownObjectException
 from gitlab import Gitlab, GitlabAuthenticationError, GitlabError
 from gitlab.v4.objects import ProjectMergeRequest
-from giturlparse import GitUrlParsed, parse
-from typing_extensions import Protocol, TypedDict
 
 from patchwork.logger import logger
 
 
-def get_slug_from_remote_url(remote_url: str) -> str:
-    parsed_repo: GitUrlParsed = parse(remote_url)
-    parts = [parsed_repo.owner, *parsed_repo.groups, parsed_repo.name]
-    slug = "/".join(parts)
-    return slug
-
-
-@define
+@dataclass(slots=True)
 class Comment:
     path: str
     body: str
@@ -34,41 +20,10 @@ class Comment:
     end_line: int
 
 
-class IssueText(TypedDict):
-    title: str
-    body: str
-    comments: list[str]
-
-
-class PullRequestComment(TypedDict):
-    user: str
-    body: str
-
-
-class PullRequestTexts(TypedDict):
-    title: str
-    body: str
-    comments: list[PullRequestComment]
-    diffs: dict[str, str]
-
-
-class PullRequestState(Enum):
-    OPEN = (["open"], ["opened"])
-    CLOSED = (["closed"], ["closed", "merged"])
-
-    def __init__(self, github_state: list[str], gitlab_state: list[str]):
-        self.github_state: list[str] = github_state
-        self.gitlab_state: list[str] = gitlab_state
-
-
 _COMMENT_MARKER = "<!-- PatchWork comment marker -->"
 
 
 class PullRequestProtocol(Protocol):
-    @property
-    def id(self) -> int:
-        ...
-
     def url(self) -> str:
         ...
 
@@ -83,7 +38,7 @@ class PullRequestProtocol(Protocol):
     def reset_comments(self) -> None:
         ...
 
-    def texts(self) -> PullRequestTexts:
+    def file_diffs(self) -> dict[str, str]:
         ...
 
     @staticmethod
@@ -153,10 +108,10 @@ class ScmPlatformClientProtocol(Protocol):
     def get_slug_and_id_from_url(self, url: str) -> tuple[str, int] | None:
         ...
 
-    def find_issue_by_url(self, url: str) -> IssueText | None:
+    def find_issue_by_url(self, url: str) -> list[str] | None:
         ...
 
-    def find_issue_by_id(self, slug: str, issue_id: int) -> IssueText | None:
+    def find_issue_by_id(self, slug: str, issue_id: int) -> list[str] | None:
         ...
 
     def get_pr_by_url(self, url: str) -> PullRequestProtocol | None:
@@ -165,14 +120,7 @@ class ScmPlatformClientProtocol(Protocol):
     def find_pr_by_id(self, slug: str, pr_id: int) -> PullRequestProtocol | None:
         ...
 
-    def find_prs(
-        self,
-        slug: str,
-        state: PullRequestState | None = None,
-        original_branch: str | None = None,
-        feature_branch: str | None = None,
-        limit: int | None = None,
-    ) -> list[PullRequestProtocol]:
+    def find_pr(self, slug: str, original_branch: str, feature_branch: str) -> PullRequestProtocol | None:
         ...
 
     def create_pr(
@@ -195,10 +143,6 @@ class GitlabMergeRequest(PullRequestProtocol):
     def __init__(self, mr: ProjectMergeRequest):
         self._mr = mr
 
-    @property
-    def id(self) -> int:
-        return self._mr.iid
-
     def url(self) -> str:
         return self._mr.web_url
 
@@ -214,29 +158,23 @@ class GitlabMergeRequest(PullRequestProtocol):
             note = self._mr.notes.create({"body": final_body})
             return f"#note_{note.get_id()}"
 
-        commit = None
-        for i in range(3):
+        while True:
             try:
                 commit = self._mr.commits().next()
             except StopIteration:
-                time.sleep(2**i)
+                continue
 
-        if commit is None:
-            return None
+            break
 
-        diff = None
-        for i in range(3):
+        while True:
             try:
                 iterator = self._mr.diffs.list(iterator=True)
                 diff = iterator.next()  # type: ignore
-                if diff.head_commit_sha == commit.get_id():
-                    break
             except StopIteration:
-                time.sleep(2**i)
                 continue
 
-        if diff is None:
-            return None
+            if diff.head_commit_sha == commit.get_id():
+                break
 
         base_commit = diff.base_commit_sha
         head_commit = diff.head_commit_sha
@@ -272,36 +210,19 @@ class GitlabMergeRequest(PullRequestProtocol):
                 if note["body"].startswith(_COMMENT_MARKER):
                     discussion.notes.delete(note["id"])
 
-    def texts(self) -> PullRequestTexts:
-        title = self._mr.title
-        body = self._mr.description
-        notes = [
-            dict(user=note.author.get("username") or "", body=note.body)
-            for note in self._mr.notes.list(iterator=True)
-            if note.system is False and note.author is not None and note.body is not None
-        ]
-
+    def file_diffs(self) -> dict[str, str]:
         diffs = self._mr.diffs.list()
         latest_diff = max(diffs, key=lambda diff: diff.created_at, default=None)
         if latest_diff is None:
-            return dict(title=title, body=body, comments=notes, diffs={})
+            return {}
 
         files = self._mr.diffs.get(latest_diff.id).diffs
-        return dict(
-            title=title,
-            body=body,
-            comments=notes,
-            diffs={file["new_path"]: file["diff"] for file in files if not file["diff"].startswith("Binary files")},
-        )
+        return {file["new_path"]: file["diff"] for file in files}
 
 
 class GithubPullRequest(PullRequestProtocol):
     def __init__(self, pr: PullRequest):
-        self._pr: PullRequest = pr
-
-    @property
-    def id(self) -> int:
-        return self._pr.number
+        self._pr = pr
 
     def url(self) -> str:
         return self._pr.html_url
@@ -333,24 +254,9 @@ class GithubPullRequest(PullRequestProtocol):
             if comment.body.startswith(_COMMENT_MARKER):
                 comment.delete()
 
-    def texts(self) -> PullRequestTexts:
-        comments = []
-        for comment in chain(self._pr.get_review_comments(), self._pr.get_issue_comments()):
-            try:
-                # Copilot user throws here
-                user = comment.user.name
-            except UnknownObjectException:
-                user = comment.user.login
-
-            comments.append(dict(user=user, body=comment.body))
-
-        return dict(
-            title=self._pr.title or "",
-            body=self._pr.body or "",
-            comments=comments,
-            # None checks for binary files
-            diffs={file.filename: file.patch for file in self._pr.get_files() if file.patch is not None},
-        )
+    def file_diffs(self) -> dict[str, str]:
+        files = self._pr.get_files()
+        return {file.filename: file.patch for file in files}
 
 
 class GithubClient(ScmPlatformClientProtocol):
@@ -387,19 +293,17 @@ class GithubClient(ScmPlatformClientProtocol):
 
         return slug, resource_id
 
-    def find_issue_by_url(self, url: str) -> IssueText | None:
+    def find_issue_by_url(self, url: str) -> list[str] | None:
         slug, issue_id = self.get_slug_and_id_from_url(url)
         return self.find_issue_by_id(slug, issue_id)
 
-    def find_issue_by_id(self, slug: str, issue_id: int) -> IssueText | None:
+    def find_issue_by_id(self, slug: str, issue_id: int) -> list[str] | None:
         repo = self.github.get_repo(slug)
         try:
             issue = repo.get_issue(issue_id)
-            return dict(
-                title=issue.title,
-                body=issue.body,
-                comments=[issue_comment.body for issue_comment in issue.get_comments()],
-            )
+            body = issue.body
+            comments = [issue_comment.body for issue_comment in issue.get_comments()]
+            return [body] + comments
         except GithubException as e:
             logger.warn(f"Failed to get issue: {e}")
             return None
@@ -417,43 +321,14 @@ class GithubClient(ScmPlatformClientProtocol):
             logger.warn(f"Failed to get PR: {e}")
             return None
 
-    def find_prs(
-        self,
-        slug: str,
-        state: PullRequestState | None = None,
-        original_branch: str | None = None,
-        feature_branch: str | None = None,
-        limit: int | None = None,
-    ) -> list[GithubPullRequest]:
+    def find_pr(self, slug, original_branch: str, feature_branch: str) -> PullRequestProtocol | None:
         repo = self.github.get_repo(slug)
-        kwargs_list = dict(state=[None], target_branch=[None], source_branch=[None])
+        pages = repo.get_pulls(base=original_branch, head=feature_branch)
+        for pr in iter(pages):
+            if pr.base.ref == original_branch and pr.head.ref == feature_branch:
+                return GithubPullRequest(pr)
 
-        if state is not None:
-            kwargs_list["state"] = state.github_state  # type: ignore
-        if original_branch is not None:
-            kwargs_list["base"] = [original_branch]  # type: ignore
-        if feature_branch is not None:
-            kwargs_list["head"] = [feature_branch]  # type: ignore
-
-        page_list = []
-        keys = kwargs_list.keys()
-        for instance in itertools.product(*kwargs_list.values()):
-            kwargs = dict(((key, value) for key, value in zip(keys, instance) if value is not None))
-            pages = repo.get_pulls(**kwargs)
-            page_list.append(pages)
-
-        branch_checker = lambda pr: True
-        if original_branch is not None:
-            branch_checker = lambda pr: branch_checker and pr.base.ref == original_branch
-        if feature_branch is not None:
-            branch_checker = lambda pr: branch_checker and pr.head.ref == feature_branch
-
-        # filter out PRs that are not the ones we are looking for
-        rv_list = []
-        for pr in itertools.islice(itertools.chain(*page_list), limit):
-            if branch_checker(pr):
-                rv_list.append(GithubPullRequest(pr))
-        return rv_list
+        return None
 
     def create_pr(
         self,
@@ -516,19 +391,17 @@ class GitlabClient(ScmPlatformClientProtocol):
 
         return slug, resource_id
 
-    def find_issue_by_url(self, url: str) -> IssueText | None:
+    def find_issue_by_url(self, url: str) -> list[str] | None:
         slug, issue_id = self.get_slug_and_id_from_url(url)
         return self.find_issue_by_id(slug, issue_id)
 
-    def find_issue_by_id(self, slug: str, issue_id: int) -> IssueText | None:
+    def find_issue_by_id(self, slug: str, issue_id: int) -> list[str] | None:
         project = self.gitlab.projects.get(slug)
         try:
             issue = project.issues.get(issue_id)
-            return dict(
-                title=issue.get("title", ""),
-                body=issue.get("description", ""),
-                comments=[note["body"] for note in issue.notes.list()],
-            )
+            body = issue["description"]
+            comments = [note["body"] for note in issue.notes.list()]
+            return [body] + comments
         except GitlabError as e:
             logger.warn(f"Failed to get issue: {e}")
             return None
@@ -546,36 +419,22 @@ class GitlabClient(ScmPlatformClientProtocol):
             logger.warn(f"Failed to get MR: {e}")
             return None
 
-    def find_prs(
-        self,
-        slug: str,
-        state: PullRequestState | None = None,
-        original_branch: str | None = None,
-        feature_branch: str | None = None,
-        limit: int | None = None,
-    ) -> list[PullRequestProtocol]:
+    def find_pr(self, slug, original_branch: str, feature_branch: str) -> PullRequestProtocol | None:
         project = self.gitlab.projects.get(slug)
-        kwargs_list = dict(iterator=[True], state=[None], target_branch=[None], source_branch=[None])
+        mrs = project.mergerequests.list(
+            iterator=True,
+            state="opened",
+            source_branch=feature_branch,
+            target_branch=original_branch,
+        )
 
-        if state is not None:
-            kwargs_list["state"] = state.gitlab_state  # type: ignore
-        if original_branch is not None:
-            kwargs_list["target_branch"] = [original_branch]  # type: ignore
-        if feature_branch is not None:
-            kwargs_list["source_branch"] = [feature_branch]  # type: ignore
+        mr: ProjectMergeRequest
+        try:
+            mr = mrs.next()  # type: ignore
+        except StopIteration:
+            return None
 
-        page_list = []
-        keys = kwargs_list.keys()
-        for instance in itertools.product(*kwargs_list.values()):
-            kwargs = dict(((key, value) for key, value in zip(keys, instance) if value is not None))
-            mrs_instance = project.mergerequests.list(**kwargs)
-            page_list.append(list(mrs_instance))
-
-        rv_list = []
-        for mr in itertools.islice(itertools.chain(*page_list), limit):
-            rv_list.append(GitlabMergeRequest(mr))
-
-        return rv_list
+        return GitlabMergeRequest(mr)
 
     def create_pr(
         self,
@@ -593,7 +452,6 @@ class GitlabClient(ScmPlatformClientProtocol):
                 "target_branch": original_branch,
                 "title": title,
                 "description": body,
-                "labels": "patchwork",
             }
         )
         mr = GitlabMergeRequest(gl_mr)  # type: ignore
