@@ -12,8 +12,7 @@ class AnalyzeImpactExtractedResponse(TypedDict):
 
 
 class AnalyzeImpactOutputs(TypedDict):
-    prompt_values: Iterable["AnalyzeImpactImpact"]
-    files_to_patch: Iterable["AnalyzeImpactImpact"]
+    impacts: Iterable["AnalyzeImpactImpact"]
 
 
 class AnalyzeImpactImpact(TypedDict):
