@@ -1,18 +1,22 @@
-from __future__ import annotations
-
+import json
 import os
 import sys
 from collections import defaultdict
 from enum import IntEnum
 from pathlib import Path
+from typing import Any
 from urllib.parse import urlparse
 
-from typing_extensions import Any
-
-from patchwork.common.context_strategy.context_strategies import ContextStrategies
-from patchwork.common.utils.utils import count_openai_tokens, open_with_chardet
+from patchwork.common.utils import (
+    count_openai_tokens,
+    defered_temp_file,
+    open_with_chardet,
+)
 from patchwork.logger import logger
-from patchwork.step import Step, StepStatus
+from patchwork.step import Step
+from patchwork.steps.ExtractCode.context_strategy.context_strategies import (
+    ContextStrategies,
+)
 
 
 def get_source_code_context(
@@ -21,19 +25,15 @@ def get_source_code_context(
     context_strategies = ContextStrategies.get_context_strategies(*ContextStrategies.ALL)
     context_strategies = [strategy for strategy in context_strategies if strategy.is_file_supported(uri, source_lines)]
     for context_strategy in context_strategies:
-        position = context_strategy.get_context_indexes(source_lines, start_line, end_line)
-        if position is None:
-            logger.debug(f'Context Strategy: "{context_strategy.__class__.__name__}" failed to return context')
+        context_start, context_end = context_strategy.get_context_indexes(source_lines, start_line, end_line)
+        if context_start is None or context_end is None:
+            logger.info(f'Context Strategy: "{context_strategy.__class__.__name__}" failed to return context')
             continue
 
-        logger.debug(f'"{context_strategy.__class__.__name__}" Context Strategy used: {position.start}, {position.end}')
-        context = "".join(source_lines[position.start : position.end])
+        logger.info(f'"{context_strategy.__class__.__name__}" Context Strategy used: {context_start}, {context_end}')
+        context = "".join(source_lines[context_start:context_end])
         if count_openai_tokens(context) <= context_token_length:
-            return position.start, position.end
-        else:
-            logger.warn(
-                f"The selected context is larger than the contex_size limit of {context_token_length}. You can increase the context_size and try again. "
-            )
+            return context_start, context_end
 
     return None, None
 
@@ -45,7 +45,7 @@ def parse_sarif_location(base_path: Path, location_str: str) -> Path | None:
         return None
 
     path = Path(uri.path)
-    if not os.path.relpath(path.resolve(), base_path.resolve()).startswith(".."):
+    if path.is_relative_to(base_path):
         return path
 
     path = str(path).lstrip("/")
@@ -177,9 +177,6 @@ def get_severity(result, reporting_descriptors):
 def transform_sarif_results(
     sarif_data: dict, base_path: Path, context_length: int, vulnerability_limit: int, severity_threshold: Severity
 ) -> dict[tuple[str, int, int, int], list[str]]:
-    total_results = len([1 for run in sarif_data.get("runs", []) for result in run.get("results", [])])
-    logger.info(f"Found {total_results} results from SARIF data")
-
     # Process each result in SARIF data
     grouped_messages = defaultdict(list)
     vulnerability_count = 0
@@ -215,13 +212,10 @@ def transform_sarif_results(
                 start_line = start_line - 1
 
                 # Generate file path assuming code is in the current working directory
-                if uri.is_absolute():
-                    file_path = str(uri.relative_to(base_path))
-                else:
-                    file_path = str(uri)
+                file_path = str(uri.relative_to(base_path))
 
                 # Extract lines from the code file
-                logger.debug(f"Extracting context for {file_path} at {start_line}:{end_line}")
+                logger.info(f"Extracting context for {file_path} at {start_line}:{end_line}")
                 try:
                     with open_with_chardet(file_path, "r") as file:
                         src = file.read()
@@ -239,32 +233,18 @@ def transform_sarif_results(
                     context_start = None
                     context_end = None
                     source_code_context = None
-                    logger.debug(f"File not found in the current working directory: {file_path}")
-                except Exception as e:
-                    context_start = None
-                    context_end = None
-                    source_code_context = None
-                    logger.error(f"Error reading file: {file_path}", e)
+                    logger.info(f"File not found in the current working directory: {file_path}")
 
                 if source_code_context is None:
-                    logger.debug(f"No context found for {file_path} at {start_line}:{end_line}")
+                    logger.info(f"No context found for {file_path} at {start_line}:{end_line}")
                     continue
 
-                fix_messages = ""
-                fix_message_delimiter = "\n- "
-                for fix in result.get("fixes", []):
-                    fix_msg = fix.get("description", {}).get("text", "")
-                    fix_messages = fix_messages + fix_message_delimiter + fix_msg
-
                 start = context_start if context_start is not None else start_line
                 end = context_end if context_end is not None else end_line
-                msg = f"Issue Description: {result.get('message', {}).get('text', '')}"
-                if fix_messages.strip() != "":
-                    msg = (
-                        f"{msg}\n"
-                        f"Suggested fixes:{fix_messages}"
-                    )
-                grouped_messages[(file_path, start, end, source_code_context)].append(msg)
+
+                grouped_messages[(file_path, start, end, source_code_context)].append(
+                    result.get("message", {}).get("text", "")
+                )
 
                 vulnerability_count = vulnerability_count + 1
                 if 0 < vulnerability_limit <= vulnerability_count:
@@ -274,42 +254,57 @@ def transform_sarif_results(
 
 
 class ExtractCode(Step):
-    required_keys = {"sarif_values"}
+    required_keys = {"sarif_file_path"}
 
     def __init__(self, inputs: dict):
-        super().__init__(inputs)
+        logger.info(f"Run started {self.__class__.__name__}")
+
         if not all(key in inputs.keys() for key in self.required_keys):
             raise ValueError(f'Missing required data: "{self.required_keys}"')
 
-        self.sarif_data = inputs["sarif_values"]
+        # Validate and set the SARIF file path
+        self.sarif_file_path = Path(inputs["sarif_file_path"])
+        if not self.sarif_file_path.exists() or not self.sarif_file_path.is_file():
+            raise ValueError(f'SARIF file path does not exist or is not a file: "{self.sarif_file_path}"')
+
         # Check and set number of lines to extract
         self.context_length = inputs.get("context_size", 1000)
         self.vulnerability_limit = inputs.get("vulnerability_limit", 10)
         self.severity_threshold = Severity.from_str(inputs.get("severity", "UNKNOWN"))
 
+        # Prepare for data extraction
+        self.extracted_code_contexts = []
+
     def run(self) -> dict:
+        # Load SARIF data
+        with open_with_chardet(self.sarif_file_path, "r") as file:
+            sarif_data = json.load(file)
+
         base_path = Path.cwd()
 
         grouped_messages = transform_sarif_results(
-            self.sarif_data, base_path, self.context_length, self.vulnerability_limit, self.severity_threshold
+            sarif_data, base_path, self.context_length, self.vulnerability_limit, self.severity_threshold
         )
 
-        if len(grouped_messages) == 0:
-            self.set_status(StepStatus.SKIPPED, "No vulnerabilities found")
-            return dict(files_to_patch=[])
-
-        if len(grouped_messages) == self.vulnerability_limit:
-            logger.debug(f"Taking {self.vulnerability_limit} vulnerability because vulnerability limit reached")
-
-        extracted_code_contexts = [
+        self.extracted_code_contexts = [
             {
                 "uri": str(file_path),
                 "startLine": start,
                 "endLine": end,
                 "affectedCode": context,
-                "messageText": "\n\n".join(msgs),
+                "messageText": "\n".join(msgs),
             }
             for (file_path, start, end, context), msgs in grouped_messages.items()
         ]
 
-        return dict(files_to_patch=extracted_code_contexts)
+        # Save extracted data to JSON
+        with defered_temp_file("w", suffix=".json") as fp:
+            json.dump(self.extracted_code_contexts, fp, indent=2)
+            output_file = Path(fp.name)
+
+        logger.info(f"Run completed {self.__class__.__name__}")
+
+        return dict(
+            code_file=output_file,
+            prompt_values=self.extracted_code_contexts,
+        )
